@@ -101,8 +101,13 @@ struct _MateBG {
 	char		*filename;
 	MateBGPlacement	 placement;
 	MateBGColorType	 color_type;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GdkRGBA	 	 primary;
+	GdkRGBA	 	 secondary;
+#else
 	GdkColor	 primary;
 	GdkColor	 secondary;
+#endif
 	gboolean	 is_enabled;
 
 	GFileMonitor* file_monitor;
@@ -143,7 +148,12 @@ static GdkPixmap       *make_root_pixmap     (GdkWindow  *window,
                                               gint        height);
 
 /* Pixbuf utils */
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void       pixbuf_average_value (GdkPixbuf  *pixbuf,
+                                        GdkRGBA    *result);
+#else
 static guint32    pixbuf_average_value (GdkPixbuf  *pixbuf);
+#endif
 static GdkPixbuf *pixbuf_scale_to_fit  (GdkPixbuf  *src,
 					int         max_width,
 					int         max_height);
@@ -153,8 +163,13 @@ static GdkPixbuf *pixbuf_scale_to_min  (GdkPixbuf  *src,
 
 static void       pixbuf_draw_gradient (GdkPixbuf    *pixbuf,
 					gboolean      horizontal,
+#if GTK_CHECK_VERSION (3, 0, 0)
+					GdkRGBA     *c1,
+					GdkRGBA     *c2,
+#else
 					GdkColor     *c1,
 					GdkColor     *c2,
+#endif
 					GdkRectangle *rect);
 
 static void       pixbuf_tile          (GdkPixbuf  *src,
@@ -206,12 +221,35 @@ static FileSize   *find_best_size      (GSList                *sizes,
 					gint                   width,
 					gint                   height);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void
+color_from_string (const char *string,
+		   GdkRGBA   *colorp)
+{
+	/* If all else fails use black */
+	gdk_rgba_parse (colorp, "#000000");
+
+	if (!string)
+		return;
+
+	gdk_rgba_parse (colorp, string);
+}
+
+static char *
+color_to_string (const GdkRGBA *color)
+{
+	return g_strdup_printf ("#%02x%02x%02x",
+				((guint) (color->red * 65535)) >> 8,
+				((guint) (color->green * 65535)) >> 8,
+				((guint) (color->blue * 65535)) >> 8);
+}
+#else
 static void
 color_from_string (const char *string,
 		   GdkColor   *colorp)
 {
 	/* If all else fails use black */
-	gdk_color_parse ("black", colorp);
+	gdk_color_parse ("#000000", colorp);
 
 	if (!string)
 		return;
@@ -227,6 +265,7 @@ color_to_string (const GdkColor *color)
 				color->green >> 8,
 				color->blue >> 8);
 }
+#endif
 
 static gboolean
 do_changed (MateBG *bg)
@@ -345,7 +384,11 @@ mate_bg_load_from_gsettings (MateBG    *bg,
 	char    *tmp;
 	char    *filename;
 	MateBGColorType ctype;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GdkRGBA c1, c2;
+#else
 	GdkColor c1, c2;
+#endif
 	MateBGPlacement placement;
 
 	g_return_if_fail (MATE_IS_BG (bg));
@@ -533,15 +576,25 @@ mate_bg_new (void)
 void
 mate_bg_set_color (MateBG *bg,
 		    MateBGColorType type,
+#if GTK_CHECK_VERSION (3, 0, 0)
+		    GdkRGBA *primary,
+		    GdkRGBA *secondary)
+#else
 		    GdkColor *primary,
 		    GdkColor *secondary)
+#endif
 {
 	g_return_if_fail (bg != NULL);
 	g_return_if_fail (primary != NULL);
 
 	if (bg->color_type != type			||
+#if GTK_CHECK_VERSION (3, 0, 0)
+	    !gdk_rgba_equal (&bg->primary, primary)			||
+	    (secondary && !gdk_rgba_equal (&bg->secondary, secondary))) {
+#else
 	    !gdk_color_equal (&bg->primary, primary)			||
 	    (secondary && !gdk_color_equal (&bg->secondary, secondary))) {
+#endif
 
 		bg->color_type = type;
 		bg->primary = *primary;
@@ -577,8 +630,13 @@ mate_bg_get_placement (MateBG *bg)
 void
 mate_bg_get_color (MateBG		*bg,
 		   MateBGColorType	*type,
+#if GTK_CHECK_VERSION (3, 0, 0)
+		   GdkRGBA		*primary,
+		   GdkRGBA		*secondary)
+#else
 		   GdkColor		*primary,
 		   GdkColor		*secondary)
+#endif
 {
 	g_return_if_fail (bg != NULL);
 
@@ -820,9 +878,15 @@ draw_color_area (MateBG       *bg,
 	switch (bg->color_type) {
 	case MATE_BG_COLOR_SOLID:
 		/* not really a big deal to ignore the area of interest */
+#if GTK_CHECK_VERSION (3, 0, 0)
+		pixel = (((guint) bg->primary.red * 65535) >> 8) << 24      |
+			(((guint) bg->primary.green * 65535) >> 8) << 24    |
+			(((guint) bg->primary.blue * 65535) >> 8) << 24      |
+#else
 		pixel = ((bg->primary.red >> 8) << 24)      |
 			((bg->primary.green >> 8) << 16)    |
 			((bg->primary.blue >> 8) << 8)      |
+#endif
 			(0xff);
 
 		gdk_pixbuf_fill (dest, pixel);
@@ -1188,7 +1252,11 @@ mate_bg_create_pixmap  (MateBG      *bg,
 
 	cr = cairo_create (surface);
 	if (!bg->filename && bg->color_type == MATE_BG_COLOR_SOLID) {
+#if GTK_CHECK_VERSION (3, 0, 0)
+		gdk_cairo_set_source_rgba (cr, &(bg->primary));
+#else
 		gdk_cairo_set_source_color (cr, &(bg->primary));
+#endif
 	}
 	else
 	{
@@ -1217,7 +1285,11 @@ mate_bg_is_dark (MateBG *bg,
 		  int      width,
 		  int      height)
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GdkRGBA color;
+#else
 	GdkColor color;
+#endif
 	int intensity;
 	GdkPixbuf *pixbuf;
 
@@ -1232,11 +1304,22 @@ mate_bg_is_dark (MateBG *bg,
 	}
 	pixbuf = get_pixbuf_for_size (bg, -1, width, height);
 	if (pixbuf) {
+#if GTK_CHECK_VERSION (3, 0, 0)
+		GdkRGBA argb;
+		guchar a, r, g, b;
+
+		pixbuf_average_value (pixbuf, &argb);
+		a = argb.alpha * 0xff;
+		r = argb.red * 0xff;
+		g = argb.green * 0xff;
+		b = argb.blue * 0xff;
+#else
 		guint32 argb = pixbuf_average_value (pixbuf);
 		guchar a = (argb >> 24) & 0xff;
 		guchar r = (argb >> 16) & 0xff;
 		guchar g = (argb >>  8) & 0xff;
 		guchar b = (argb >>  0) & 0xff;
+#endif
 
 		color.red = (color.red * (0xFF - a) + r * 0x101 * a) / 0xFF;
 		color.green = (color.green * (0xFF - a) + g * 0x101 * a) / 0xFF;
@@ -1244,9 +1327,9 @@ mate_bg_is_dark (MateBG *bg,
 		g_object_unref (pixbuf);
 	}
 
-	intensity = (color.red * 77 +
-		     color.green * 150 +
-		     color.blue * 28) >> 16;
+	intensity = ((guint) (color.red * 65535) * 77 +
+		     (guint) (color.green * 65535) * 150 +
+		     (guint) (color.blue * 65535) * 28) >> 16;
 
 	return intensity < 160; /* biased slightly to be dark */
 }
@@ -2487,8 +2570,14 @@ clear_cache (MateBG *bg)
 }
 
 /* Pixbuf utilities */
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void
+pixbuf_average_value (GdkPixbuf *pixbuf,
+                      GdkRGBA   *result)
+#else
 static guint32
 pixbuf_average_value (GdkPixbuf *pixbuf)
+#endif
 {
 	guint64 a_total, r_total, g_total, b_total;
 	guint row, column;
@@ -2497,6 +2586,9 @@ pixbuf_average_value (GdkPixbuf *pixbuf)
 	int r, g, b, a;
 	guint64 dividend;
 	guint width, height;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdouble dd;
+#endif
 
 	width = gdk_pixbuf_get_width (pixbuf);
 	height = gdk_pixbuf_get_height (pixbuf);
@@ -2542,11 +2634,20 @@ pixbuf_average_value (GdkPixbuf *pixbuf)
 		dividend = height * width;
 		a_total = dividend * 0xFF;
 	}
+#if GTK_CHECK_VERSION (3, 0, 0)
+
+	dd = dividend * 0xFF;
+	result->alpha = a_total / dd;
+	result->red = r_total / dd;
+	result->green = g_total / dd;
+	result->blue = b_total / dd;
+#else
 
 	return ((a_total + dividend / 2) / dividend) << 24
 		| ((r_total + dividend / 2) / dividend) << 16
 		| ((g_total + dividend / 2) / dividend) << 8
 		| ((b_total + dividend / 2) / dividend);
+#endif
 }
 
 static GdkPixbuf *
@@ -2602,8 +2703,13 @@ pixbuf_scale_to_min (GdkPixbuf *src, int min_width, int min_height)
 }
 
 static guchar *
+#if GTK_CHECK_VERSION (3, 0, 0)
+create_gradient (const GdkRGBA *primary,
+		 const GdkRGBA *secondary,
+#else
 create_gradient (const GdkColor *primary,
 		 const GdkColor *secondary,
+#endif
 		 int	         n_pixels)
 {
 	guchar *result = g_malloc (n_pixels * 3);
@@ -2612,9 +2718,15 @@ create_gradient (const GdkColor *primary,
 	for (i = 0; i < n_pixels; ++i) {
 		double ratio = (i + 0.5) / n_pixels;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+		result[3 * i + 0] = (guchar) ((primary->red * (1 - ratio) + secondary->red * ratio) * 0x100);
+		result[3 * i + 1] = (guchar) ((primary->green * (1 - ratio) + secondary->green * ratio) * 0x100);
+		result[3 * i + 2] = (guchar) ((primary->blue * (1 - ratio) + secondary->blue * ratio) * 0x100);
+#else
 		result[3 * i + 0] = ((guint16) (primary->red * (1 - ratio) + secondary->red * ratio)) >> 8;
 		result[3 * i + 1] = ((guint16) (primary->green * (1 - ratio) + secondary->green * ratio)) >> 8;
 		result[3 * i + 2] = ((guint16) (primary->blue * (1 - ratio) + secondary->blue * ratio)) >> 8;
+#endif
 	}
 
 	return result;
@@ -2623,8 +2735,13 @@ create_gradient (const GdkColor *primary,
 static void
 pixbuf_draw_gradient (GdkPixbuf    *pixbuf,
 		      gboolean      horizontal,
+#if GTK_CHECK_VERSION (3, 0, 0)
+		      GdkRGBA      *primary,
+		      GdkRGBA      *secondary,
+#else
 		      GdkColor     *primary,
 		      GdkColor     *secondary,
+#endif
 		      GdkRectangle *rect)
 {
 	int width;
